import os
import networkx as nx
import numpy as np
import scipy as sp
from scipy import misc as spmi
import matplotlib.pyplot as plt
import sys



# specify the gold standard network and the directory with query networks
gold_file = sys.argv[1] 
indir = sys.argv[2]

queries=[]
for root, dirs, filenames in os.walk(indir):
	for f in filenames:
		if not ".count" in f: 
			queries.append(os.path.join(root, f))







def get_egos(network_file):
	G = nx.read_edgelist(network_file)
	G=nx.Graph(G)
	loops=G.selfloop_edges()
	G.remove_edges_from(loops)
	n=G.number_of_nodes()
	m=G.number_of_edges()
	
	for x in sorted(G.nodes()):
		egonodes=set()
		egonodes.add(x)
		egonodes.update(G.neighbors(x))
		for i in G.neighbors(x):
			egonodes.update(G.neighbors(i))
		ego=G.subgraph(egonodes)
		n=ego.number_of_nodes()
		m=ego.number_of_edges()
		f = open(x+'_temp.in','a')
		print >>f,n,m
		mapper={}
		intmap=0
		for i in nx.nodes(ego):
			mapper[str(i)] = intmap;
			intmap=intmap+1

		for e in nx.edges(ego):
			print >>f,mapper[e[0]],mapper[e[1]]
		f.close()
		os.system('./orca 4 '+x+'_temp.in '+network_file+'.count '+str(n)+' '+str(m));
		os.system('rm '+x+'_temp.in')
	return 1


#Get subgraph-counts for all ego-networks of gold-standard and query networks
get_egos(gold_file)
for i in queries:
	get_egos(i)

#Get ego-graph densities from the gold standard
(name_gold,nodes_gold,edges_gold)=([],[],[])
gold=[[] for i in range(9)]
f = open(gold_file+'.count','r')
A=f.readlines()
f.close()

for line in A:
	line=line.strip()
	line=line.split(';')
	name_gold.append(line[0])
	nodes_gold.append(line[1])
	edges_gold.append(line[2])
	for k in range(1,9):
		gold[k].append(line[(k+2)])

graph_den_gold=np.zeros(len(nodes_gold))

for i in range(0,len(nodes_gold)):
	if (int(nodes_gold[i])>=2):
		graph_den_gold[i]=float(edges_gold[i]) / float(spmi.comb(int(nodes_gold[i]),2))
	else:
		graph_den_gold[i]=0
	

# binning ego-networks of gold-standard by density

(n, bins, patches)=plt.hist(graph_den_gold, bins=100, range=(0,1))
counts=1
while not(counts==0):
	len(n)
	counts=0
	for i in range(0+1,len(n)-1):
		if n[i]<5:
			if n[i-1]<n[i+1]:
				bins=np.delete(bins,[i])
				counts=counts+1
				break
			elif n[i-1]>n[i+1]:
				bins=np.delete(bins,[i+1])
				counts=counts+1
				break
			elif n[i-1]==n[i+1]:
				bins=np.delete(bins,[i])
				counts=counts+1
				break
	
	(n, bins, patches)=plt.hist(graph_den_gold, bins=bins)

if n[0]<5:
	bins=np.delete(bins,[1])

(n, bins, patches)=plt.hist(graph_den_gold, bins=bins)


while n[len(n)-1]<5:
	bins=np.delete(bins,[len(bins)-2])
	(n, bins, patches)=plt.hist(graph_den_gold, bins=bins)




num_egos_gold={}
all_expected={}

for o in range(0,len(bins)-1):	
	#make expected for this graph den
	expected=np.zeros(9)
	num_egos_gold[str(bins[o+1])]=0
	for i in range(0,len(nodes_gold)):
		if (graph_den_gold[i]>=bins[o]) and (graph_den_gold[i]<bins[o+1]) and (float(nodes_gold[i])>=5):
			for k in range(1,3):
				expected[k]=float(expected[k])+(float(gold[k][i])/float(spmi.comb(int(nodes_gold[i]),3)))
			for k in range(3,9):
				expected[k]=float(expected[k])+(float(gold[k][i])/float(spmi.comb(int(nodes_gold[i]),4)))
			num_egos_gold[str(bins[o+1])]=num_egos_gold[str(bins[o+1])]+1
	
	for k in range(1,9):
		if float(num_egos_gold[str(bins[o+1])]) > 0:
			expected[k]=float(expected[k])/float(num_egos_gold[str(bins[o+1])])
		else:
			expected[k]=0 
	all_expected[o]=expected
	


#d_S statistic - l1 and l2 must be same lenght!!!
def distance(l1,l2):
	
	d2ss=float(0)
	for i in range(0,len(l1)):
		if ((np.sqrt(l1[i]*l1[i]+l2[i]*l2[i]))==0):
			d2ss=d2ss+0
		else:
			d2ss=d2ss+((l1[i]*l2[i])/np.sqrt(l1[i]*l1[i]+l2[i]*l2[i]))
	
	dx=float(0)
	for i in range(0,len(l1)):
		if ((np.sqrt(l1[i]*l1[i]+l2[i]*l2[i]))==0):
			dx=dx+0
		else:
			dx=dx+((l1[i]*l1[i])/np.sqrt(l1[i]*l1[i]+l2[i]*l2[i]))
	
	dy=float(0)
	for i in range(0,len(l1)):
		if ((np.sqrt(l1[i]*l1[i]+l2[i]*l2[i]))==0):
			dy=dy+0
		else:
			dy=dy+((l2[i]*l2[i])/np.sqrt(l1[i]*l1[i]+l2[i]*l2[i]))
	
	if not((np.sqrt(dx)*np.sqrt(dy))==0):
		little_d=0.5*(1-(d2ss/(np.sqrt(dx)*np.sqrt(dy))))
	elif ((np.sqrt(dx)*np.sqrt(dy))==0):
		little_d=1
	
	return little_d


#compare gold to query
def Sw_goldVSquery(query_file,query_num,bins,expected):

	(name_query,nodes_query,edges_query)=([],[],[])
	query=[[] for i in range(9)]
	f = open(query_file+'.count','r')
	A=f.readlines()
	f.close()
	for line in A:
		line=line.strip()
		line=line.split(';')
		name_query.append(line[0])
		nodes_query.append(line[1])
		edges_query.append(line[2])
		for k in range(1,9):
			query[k].append(line[(k+2)]) 
	
	graph_den_query=np.zeros(len(nodes_query))
	for i in range(0,len(nodes_query)):
		if (int(nodes_query[i])>=2):
			graph_den_query[i]=float(edges_query[i]) / float(sp.misc.comb(int(nodes_query[i]),2))
		else:
			graph_den_query[i]=0
		
	num_egos_gold={}
	num_egos_query={}
	sums_allden=np.zeros(9)
	sums_allden=np.zeros(9)
	denomi=np.zeros(9)
	
	for o in range(0,len(bins)-1):
		for i in range(0,len(nodes_query)):
			if (graph_den_query[i]>=bins[o]) and (graph_den_query[i]<bins[o+1]) and (float(nodes_query[i])>=3):
				for k in range(1,3):
					sums_allden[k]=sums_allden[k]+((float(query[k][i])-(float(sp.misc.comb(int(nodes_query[i]),3))*float(expected[o][k]))))
				for k in range(3,9):
					sums_allden[k]=sums_allden[k]+((float(query[k][i])-(float(sp.misc.comb(int(nodes_query[i]),4))*float(expected[o][k]))))
	return sums_allden


#Sw
j=0
Sws={}
for i in queries:
	print str(i)+' versus gold standard '+str(gold_file)
	Sws[j]=Sw_goldVSquery(i,j,bins,all_expected)
	Sws[j]=Sws[j][1:]
	print Sws[j]
	j=j+1


#k=3
Sw_k={}
for i in range(0,len(Sws)):
	Sw_k[i]=Sws[i][0:2]

dmat=np.zeros((len(queries),len(queries)),float)
for i in range(0,len(queries)):
	for j in range(0,len(queries)):
		dmat[i,j]=distance(Sw_k[i],Sw_k[j])
n_file='ds3'
f = open(n_file,'w')

f.write('\t')
for i in range(0,len(dmat)):
	f.write(str(queries[i])+'\t')
f.write('\n')


for i in range(0,len(dmat)):
	f.write(str(queries[i])+'\t')
	for j in range(0,len(dmat[i])):
		f.write(str(round(dmat[i,j],6))+'\t')
	f.write('\n')
f.close()



#k=4
Sw_k={}
for i in range(0,len(Sws)):
	Sw_k[i]=Sws[i][2:8]

dmat=np.zeros((len(queries),len(queries)))
for i in range(0,len(queries)):
	for j in range(0,len(queries)):
		dmat[i,j]=distance(Sw_k[i],Sw_k[j])

n_file='ds4'
f = open(n_file,'w')

f.write('\t')
for i in range(0,len(dmat)):
	f.write(str(queries[i])+'\t')
f.write('\n')


for i in range(0,len(dmat)):
	f.write(str(queries[i])+'\t')
	for j in range(0,len(dmat[i])):
		f.write(str(round(dmat[i,j],6))+'\t')
	f.write('\n')

f.close()
